#![allow(missing_docs, unused_variables, trivial_casts)]
extern crate openapi_v3;
extern crate clap;
extern crate env_logger;
extern crate futures;

// log may be unused if there are no examples
#[allow(unused_imports)]
#[macro_use]
extern crate log;
#[macro_use]
extern crate swagger;
extern crate tokio;
extern crate chrono;
#[macro_use]
extern crate error_chain;
extern crate hyper;
extern crate uuid;

#[cfg(not(any(target_os = "macos", target_os = "windows", target_os = "ios")))]
extern crate openssl;
#[cfg(not(any(target_os = "macos", target_os = "windows", target_os = "ios")))]
extern crate tokio_openssl;

mod server;

#[allow(unused_imports)]
use futures::{Future, future, Stream, stream};
#[allow(unused_imports)]
use openapi_v3::{Api, ApiNoContext, Client, ContextWrapperExt,
                      ApiError,
                      CallbackWithHeaderPostResponse,
                      EnumInPathPathParamGetResponse,
                      MandatoryRequestHeaderGetResponse,
                      MergePatchJsonGetResponse,
                      MultigetGetResponse,
                      MultipleAuthSchemeGetResponse,
                      ParamgetGetResponse,
                      ReadonlyAuthSchemeGetResponse,
                      RegisterCallbackPostResponse,
                      RequiredOctetStreamPutResponse,
                      ResponsesWithHeadersGetResponse,
                      Rfc7807GetResponse,
                      UntypedPropertyGetResponse,
                      UuidGetResponse,
                      XmlExtraPostResponse,
                      XmlOtherPostResponse,
                      XmlOtherPutResponse,
                      XmlPostResponse,
                      XmlPutResponse
                     };
use clap::{App, Arg};

// swagger::Has may be unused if there are no examples
#[allow(unused_imports)]
use swagger::{ContextBuilder, EmptyContext, XSpanIdString, Has, Push, AuthData};

// rt may be unused if there are no examples
#[allow(unused_mut)]
fn main() {
    env_logger::init();

    let matches = App::new("client")
        .arg(Arg::with_name("operation")
            .help("Sets the operation to run")
            .possible_values(&[
                "CallbackWithHeaderPost",
                "MandatoryRequestHeaderGet",
                "MergePatchJsonGet",
                "MultigetGet",
                "MultipleAuthSchemeGet",
                "ParamgetGet",
                "ReadonlyAuthSchemeGet",
                "RegisterCallbackPost",
                "RequiredOctetStreamPut",
                "ResponsesWithHeadersGet",
                "Rfc7807Get",
                "UntypedPropertyGet",
                "UuidGet",
                "XmlExtraPost",
                "XmlOtherPost",
                "XmlOtherPut",
                "XmlPost",
                "XmlPut",
            ])
            .required(true)
            .index(1))
        .arg(Arg::with_name("https")
            .long("https")
            .help("Whether to use HTTPS or not"))
        .arg(Arg::with_name("host")
            .long("host")
            .takes_value(true)
            .default_value("localhost")
            .help("Hostname to contact"))
        .arg(Arg::with_name("port")
            .long("port")
            .takes_value(true)
            .default_value("80")
            .help("Port to contact"))
        .get_matches();

    let is_https = matches.is_present("https");
    let base_url = format!("{}://{}:{}",
                           if is_https { "https" } else { "http" },
                           matches.value_of("host").unwrap(),
                           matches.value_of("port").unwrap());

    let client = if matches.is_present("https") {
        // Using Simple HTTPS
        Client::try_new_https(&base_url)
            .expect("Failed to create HTTPS client")
    } else {
        // Using HTTP
        Client::try_new_http(
            &base_url)
            .expect("Failed to create HTTP client")
    };

    let context: make_context_ty!(ContextBuilder, EmptyContext, Option<AuthData>, XSpanIdString) =
        make_context!(ContextBuilder, EmptyContext, None as Option<AuthData>, XSpanIdString::default());

    let client = client.with_context(context);

    let mut rt = tokio::runtime::Runtime::new().unwrap();

    // We could do HTTPS here, but for simplicity we don't
    rt.spawn(server::create("127.0.0.1:8081", false));

    match matches.value_of("operation") {
        Some("CallbackWithHeaderPost") => {
            let result = rt.block_on(client.callback_with_header_post(
                  "url_example".to_string()
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        /* Disabled because there's no example.
        Some("EnumInPathPathParamGet") => {
            let result = rt.block_on(client.enum_in_path_path_param_get(
                  ???
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        */
        Some("MandatoryRequestHeaderGet") => {
            let result = rt.block_on(client.mandatory_request_header_get(
                  "x_header_example".to_string()
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        Some("MergePatchJsonGet") => {
            let result = rt.block_on(client.merge_patch_json_get(
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        Some("MultigetGet") => {
            let result = rt.block_on(client.multiget_get(
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        Some("MultipleAuthSchemeGet") => {
            let result = rt.block_on(client.multiple_auth_scheme_get(
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        Some("ParamgetGet") => {
            let result = rt.block_on(client.paramget_get(
                  Some(serde_json::from_str::<uuid::Uuid>("38400000-8cf0-11bd-b23e-10b96e4ef00d").expect("Failed to parse JSON example")),
                  None,
                  None
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        Some("ReadonlyAuthSchemeGet") => {
            let result = rt.block_on(client.readonly_auth_scheme_get(
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        Some("RegisterCallbackPost") => {
            let result = rt.block_on(client.register_callback_post(
                  "url_example".to_string()
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        Some("RequiredOctetStreamPut") => {
            let result = rt.block_on(client.required_octet_stream_put(
                  swagger::ByteArray(Vec::from("BYTE_ARRAY_DATA_HERE"))
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        Some("ResponsesWithHeadersGet") => {
            let result = rt.block_on(client.responses_with_headers_get(
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        Some("Rfc7807Get") => {
            let result = rt.block_on(client.rfc7807_get(
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        Some("UntypedPropertyGet") => {
            let result = rt.block_on(client.untyped_property_get(
                  None
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        Some("UuidGet") => {
            let result = rt.block_on(client.uuid_get(
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        Some("XmlExtraPost") => {
            let result = rt.block_on(client.xml_extra_post(
                  None
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        Some("XmlOtherPost") => {
            let result = rt.block_on(client.xml_other_post(
                  None
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        Some("XmlOtherPut") => {
            let result = rt.block_on(client.xml_other_put(
                  None
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        Some("XmlPost") => {
            let result = rt.block_on(client.xml_post(
                  None
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        Some("XmlPut") => {
            let result = rt.block_on(client.xml_put(
                  None
            ));
            info!("{:?} (X-Span-ID: {:?})", result, (client.context() as &Has<XSpanIdString>).get().clone());
        },
        _ => {
            panic!("Invalid operation provided")
        }
    }
}
